/*
 *	Sherlock Gatherer -- Data Type Guess Utility
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gather.h"

int
main(int argc, char **argv)
{
  struct odes *o;
  int err;

  initlog(argv[0]);
  if (argc == 3 && !strcmp(argv[1], "-o"))
    o = obj_load(argv[2]);
  else if (argc >= 2)
    {
      o = obj_new();
      set_attr(o, 'U', argv[1]);
      set_attr(o, '@', argv[1]);
      while (argc > 2)
	{
	  byte *a = argv[2];
	  if (strlen(a) < 2 || a[1] != '=')
	    {
	      fprintf(stderr, "Attribute syntax error!\n");
	      return 1;
	    }
	  set_attr(o, a[0], a+2);
	  argc--, argv++;
	}
    }
  else
    {
      fprintf(stderr, "Usage: dtguess (-o <object> | <file> [T=<content-type>] [E=<content-encoding>] [U=<url>] [c=<charset>] [<attr>=<value>] ...)\n");
      return 1;
    }

  puts("--- INIT");
  datatype_init();
  charset_init();
  trace_data_types = 10;
  trace_charsets = 10;

  puts("--- Analysing object:");
  obj_dump(o);
  puts("");

again:
  err = datatype_guess(o);
  if (err > 0)
    {
      printf("!!! Error %d\n", err);
      return 1;
    }

  if (err < 0)
    {
      puts("\n--- Intermediate result:");
      obj_dump(o);
      puts("Decoding...");
      err = c_e_decode(o, err);
      if (err > 0)
	{
	  printf("!!! Error %d\n", err);
	  return 1;
	}
      puts("\n--- After decoding:");
      obj_dump(o);
      puts("Running datatype_guess again...");
      goto again;
    }

  puts("\n--- Result:");
  obj_dump(o);
  return 0;
}
