/*
 *	Sherlock Gatherer Queue Recovery Utility
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "gather.h"

#include <lib/url.h>

static void
got_host_obj(oid_t oid, byte *name)
{
  struct odes *o;
  struct qhost *h;

  o = obj_load(name);
  h = new_host(atoi(find_aval(o, 'p')),
	       find_aval(o, 'H'),
	       atoi(find_aval(o, 'P')),
	       oid);
  obj_free(o);
  enqueue_host(h);
}

static void
rec_queue(void)
{
  byte url[MAX_URL_SIZE];
  byte *b;
  struct urlrec *ur;

  b = urldb_get_first(url);
  while (b)
    {
      if (! (ur = urldb_lookup(url)))
	log(L_ERROR "Got lost in mist for %s", url);
      else if (ur->access & 1)		/* queued */
	{
	  byte b1[MAX_URL_SIZE], b2[MAX_URL_SIZE];
	  struct url u;
	  if (url_canon_split(url, b1, b2, &u))
	    log(L_ERROR "Error processing %s", url);
	  else
	    {
	      struct qhost *h = find_host(u.protoid, u.host, u.port);
	      if (!h)
		log(L_ERROR "No host for %s", url);
	      else
		enqueue_item(h, u.rest);
	    }
	}
      b = urldb_get_next(url);
    }
}

int
main(int argc, char **argv)
{
  initlog(argv[0]);

  log(L_INFO "Deleting old host database...");
  unlink("db/queue");
  unlink("db/hosts");

  db_init();
  gather_lock();
  urldb_open();
  queue_init();

  log(L_INFO "Recovering hosts...");
  scan_obj_tree("host", got_host_obj);

  log(L_INFO "Recovering queued URL's...");
  rec_queue();

  queue_cleanup();
  urldb_close();
  gather_unlock();
  return 0;
}
