/*
 *	Sherlock Library -- String Comparison in Usual Czech Order
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <string.h>

#ifdef TEST
#include <stdio.h>
#endif

#include "config.h"
#include "string.h"

int
Cstrcmp(const unsigned char *a, const unsigned char *b)
{
  const byte *x, *y;
  byte p, q;

#ifdef TEST
  puts("Pass 1");
#endif

  x = a;				/* Pass 1 -- by words */
  y = b;
  for(;;)
    {
      while (*x && !Cword(*x))		/* Skip non-word characters */
	x++;
      while (*y && !Cword(*y))
	y++;
      if (!*x && !*y)			/* Both at the end => go to pass 2 */
	break;
      if (!*x)
	return -1;
      if (!*y)
	return 1;
      while (Cword(*x) && Cword(*y))
	{
	  p = _c_collate[*x];
	  q = _c_collate[*y];
	  if (p < q)
	    return -1;
	  if (p > q)
	    return 1;
	  p = _c_order[*x++] >> 4U;
	  q = _c_order[*y++] >> 4U;
	  if (p < q)
	    return -1;
	  if (p > q)
	    return 1;
	}
      if (Cword(*x))
	return 1;
      if (Cword(*y))
	return -1;
    }

#ifdef TEST
  puts("Pass 2");
#endif

  x = a;				/* Pass 2 -- by words, but with collation order */
  y = b;
  for(;;)
    {
      while (*x && !Cword(*x))		/* Skip non-word characters */
	x++;
      while (*y && !Cword(*y))
	y++;
      if (!*x)				/* Both at the end => go to pass 3 (!*y implied) */
	break;				/* Must have *x && *y because it passed the first pass */
      while (Cword(*x) && Cword(*y))
	{
	  p = _c_order[*x++] & 0x0f;
	  q = _c_order[*y++] & 0x0f;
	  if (p < q)
	    return -1;
	  if (p > q)
	    return 1;
	}
      /* Must have !Cword(*x) && !Cword(*y) now as it passed the first pass */
    }

#ifdef TEST
  puts("Pass 3");
#endif

  x = a;				/* Pass 3 -- ignore only spaces */
  y = b;
  for(;;)
    {
      while (Cspace(*x))
	x++;
      while (Cspace(*y))
	y++;
      if (!*x)				/* Implies !*y */
	break;
      if (*x < *y)
	return -1;
      if (*x > *y)
	return 1;
      x++;
      y++;
    }

#ifdef TEST
  puts("Pass 4");
#endif

  return strcmp(a, b);			/* Exact character code match */
}

#ifdef TEST

void
main(void)
{
  byte x[256], y[256];

  for(;;)
    {
      fgets(x, 256, stdin);
      fgets(y, 256, stdin);
      printf("%d\n", Cstrcmp(x, y));
    }
}

#endif
