/*
 *	Sherlock Search Engine -- Lexical Analysis of Queries
 *
 *	(c) 1997 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sherlock.h"

#include <lib/string.h>

#include "parse.tab.h"

static byte *pos;

static byte *ref_keywords[] = {
  "REF", "REFURL", "REFAPPLET", "REFIMG", "REFSELF", "REFFORM", NULL
};

void
lex_init(byte *buf)
{
  pos = buf;
}

struct keywd {
  char *keyword;
  int val;
};

static struct keywd keywords[] = {
  { "SECTION",		       	SECTION	 },
  { "FROM",			FROM	 },
  { "TO",			TO	 },
  { "AND",			AND	 },
  { "OR",			OR	 },
  { "NOT",			NOT	 },
  { "ANY",			ANY	 },
  { "CONTROL",			CONTROL	 },
#ifdef DEBUG
  { "XYZZY",			XYZZY	 },
#endif
  { "ACCENTS",			ACCENTS	 },
};

int
yylex(void)
{
  while (Cspace(*pos))
    pos++;
  if (!*pos)
    return EOLN;
  if (Cdigit(*pos))
    {
      uns i = 0;
      while (Cdigit(*pos))
	{
	  i = 10*i + *pos++ - '0';
	  if (i > 1000000)
	    err("Number too large");
	}
      yylval.i = i;
      return NUM;
    }
  if (Calpha(*pos))
    {
      byte *start = pos;
      byte x;
      uns k;

      while (Calpha(*pos))
	{
	  *pos = Cupcase(*pos);
	  pos++;
	}
      x = *pos;
      *pos = 0;
      for(k=0; k<sizeof(keywords)/sizeof(keywords[0]); k++)
	if (!strcmp(start, keywords[k].keyword))
	  break;
      if (k < sizeof(keywords)/sizeof(keywords[0]))
	k = keywords[k].val;
      else
	{
	  byte **z = ref_keywords;
	  while (*z && strcmp(start, *z))
	    z++;
	  if (!*z)
	    err("Unknown keyword");
	  yylval.i = z - ref_keywords;
	  k = REF;
	}
      *pos = x;
      return k;
    }
  switch (*pos++)
    {
    case '"':
      {
	byte *z = pos;
	while (*pos && *pos != '"')
	  pos++;
	if (!*pos)
	  err("Misquoted string");
	*pos++ = 0;
	yylval.s = z;
	return STRING;
      }
    case '&':
      return AND;
    case '|':
      return OR;
    case '!':
      return NOT;
    case ',':
    case '(':
    case ')':
      return pos[-1];
    default:
      err("Unknown character");
    }
}
